@@ -1,22 +1,18 @@
-#' @importFrom httr GET write_disk
-#' @importFrom rvest html_nodes html_attrs
-#' @importFrom xml2 read_html
-ihpd_tf <- function(file = 1, regex = "hp[0-9]"){
-  relative_url <-
-    xml2::read_html("https://www.dallasfed.org/institute/houseprice#tab2") %>%
-    rvest::html_nodes("a") %>%
-    rvest::html_attr("href") %>%
-    grep(".xlsx$", ., value = TRUE) %>%
-    grep(regex, ., value = TRUE) %>%
-    magrittr::extract(file)
 
-  absolute_url <- paste0("https://www.dallasfed.org", relative_url)
 
-  tf <- tempfile(fileext = ".xlsx")
-  httr::GET(absolute_url, write_disk(tf))
-  tf
+set_source <- function(tbl, url) {
+  attr(tbl, "source") <- attr(url, "source")
+  tbl
 }
 
+set_version <- function(tbl, vers) {
+  attr(tbl, "version") <- attr(vers, "version")
+  tbl
+}
+
+
+# Raw ---------------------------------------------------------------------
+
 
 #' @importFrom dplyr mutate full_join
 #' @importFrom lubridate yq
@@ -32,9 +28,10 @@ format_excel_raw <- function(x, sheet_num, nm, ...) {
 
 #' @importFrom readxl read_excel
 #' @importFrom tidyr drop_na gather
-ihpd_get_raw <- function() {
+ihpd_get_raw <- function(.version, .access_info) {
 
-  tf <- ihpd_tf(regex = "hp[0-9]")
+  tf <- ihpd_tf(version = .version, regex = "hp[0-9]", access_info = .access_info)
+  tf %||% return(invisible(NULL))
   on.exit(file.remove(tf))
 
   list(
@@ -43,9 +40,13 @@ ihpd_get_raw <- function() {
     format_excel_raw(tf, 4, "pdi"),
     format_excel_raw(tf, 5, "rpdi")) %>%
     reduce(full_join, by = c("Date", "country")) %>%
-    drop_na()
+    drop_na() %>%
+    set_source(tf) %>%
+    set_version(.version)
 }
 
+# Bsadf -------------------------------------------------------------------
+
 
 format_excel_bsadf <- function(x, nm, nms, ...) {
   x %>% slice(-1) %>% set_names(nms) %>%
@@ -55,9 +56,10 @@ format_excel_bsadf <- function(x, nm, nms, ...) {
 }
 
 
-ihpd_get_bsadf <- function() {
+ihpd_get_bsadf <- function(.version, .access_info) {
 
-  tf <- ihpd_tf(file = 1, regex = "hpta[0-9]")
+  tf <- ihpd_tf(version = .version, regex = "hpta[0-9]", access_info = .access_info)
+  tf %||% return(invisible(NULL))
   on.exit(file.remove(tf))
 
   nms <- readxl::read_excel(tf, sheet = 2, range = "G2:AE2") %>%
@@ -89,12 +91,12 @@ ihpd_get_bsadf <- function() {
 
    full_join(tbl_lag1, tbl_lag4,
              by = c("country", "Date", "crit", "type", "lag", "value")) %>%
-     select(Date, country, type, lag, value, crit)
+     select(Date, country, type, lag, value, crit) %>%
+     set_source(tf) %>%
+     set_version(.version)
 }
 
-format_excel_sadf <- function(x) {
-  x
-}
+# GSADF -------------------------------------------------------------------
 
 format_excel_gsadf <- function(x, nm, lag = 1) {
   x %>%
@@ -104,9 +106,10 @@ format_excel_gsadf <- function(x, nm, lag = 1) {
     gather(tstat, value, -country, -type, -lag)
 }
 
-ihpd_get_gsadf <- function() {
+ihpd_get_gsadf <- function(.version, .access_info) {
 
-  tf <- ihpd_tf(regex = "hpta[0-9]")
+  tf <- ihpd_tf(version = .version, regex = "hpta[0-9]", access_info = .access_info)
+  tf %||% return(invisible(NULL))
   on.exit(file.remove(tf))
 
   nms <- readxl::read_excel(tf, sheet = 2, range = "G2:AE2") %>%
@@ -131,11 +134,15 @@ ihpd_get_gsadf <- function() {
     reduce(full_join, by = c("tstat", "country", "type", "lag", "value")) %>%
     full_join(cv, by = "tstat") %>%
     mutate_at(vars(crit, sig), as.numeric) %>%
-    select(country, type, tstat, lag, value, crit, sig)
-
+    select(country, type, tstat, lag, value, crit, sig) %>%
+    set_source(tf) %>%
+    set_version(.version)
 }
 
-#' Download International House Price Database Data
+# GET ---------------------------------------------------------------------
+
+
+#' Download Data from the International House Price Database
 #'
 #' Available downloads:
 #' \enumerate{
@@ -144,7 +151,11 @@ ihpd_get_gsadf <- function() {
 #'   \item bsadf: Backward sup ADF statistic sequence
 #'  }
 #'
-#' @param symbol which dataset to download.
+#' @param symbol Which dataset to download.
+#' @param version Which version to download. Version number should be a character
+#' of the following format %Y%q (e.g. '1801' - corresponds to year 2018,
+#' Quarter 1). Versions start from '1102'. Defaults at the latest available.
+#' @param verbose whether to print the url of the excel file that is accessing.
 #'
 #' @details
 #'
@@ -189,25 +200,35 @@ ihpd_get_gsadf <- function() {
 #'
 #' @importFrom purrr reduce
 #' @importFrom dplyr slice rename select mutate_at vars
+#'
+#' @return Returns a tibble in long format.
 #' @export
 #'
 #' @examples
 #'
 #' ihpd_get()
-ihpd_get <- function(symbol = c("raw", "gsadf", "bsadf")) {
+ihpd_get <- function(symbol = c("raw", "gsadf", "bsadf"), version = NULL,
+                     verbose = TRUE) {
   symbol <- match.arg(symbol)
-  switch(symbol,
-    raw = ihpd_get_raw(),
-    gsadf = ihpd_get_gsadf(),
-    bsadf = ihpd_get_bsadf()
-    )
+  switch(
+    symbol,
+    raw = ihpd_get_raw(.version = version, .access_info = verbose),
+    gsadf = ihpd_get_gsadf(.version = version, .access_info = verbose),
+    bsadf = ihpd_get_bsadf(.version = version, .access_info = verbose)
+  )
 }
 
+
 #' Fetches the latest release dates
 #'
-#'@export
-#'@importFrom rvest html_table
-#'@importFrom rlang set_names
+#' @export
+#' @importFrom rvest html_table
+#' @importFrom rlang set_names
+#'
+#' @return A data.frame with 2 variables with the latest release dates data.
+#'
+#' @examples
+#' ihpd_release_dates()
 ihpd_release_dates <- function() {
 
   tbl <- xml2::read_html("https://www.dallasfed.org/institute/houseprice") %>%
